@@ -31,20 +31,24 @@ private:
 };
 
 /** 
- * Abstract class for processes, active entities that need a method activate().
+ * Abstract class for processes, active entities that need a method run().
  */
 class Process: public Entity {
 public:
-  Process(Simulator* sim, std::string name, int mon, bool is_gen=false): 
-    Entity(sim, name, mon), is_gen(is_gen) {}
+  Process(Simulator* sim, std::string name, int mon, bool generator=false): 
+    Entity(sim, name, mon), generator(generator), active(true) {}
   virtual ~Process(){}
-  virtual void activate() = 0;
-  inline bool is_generator() { return is_gen; }
+  virtual void run() = 0;
+  inline virtual void activate() { active = true; }
+  inline virtual void deactivate(bool restart) { active = false; }
+  inline bool is_generator() { return generator; }
+  inline bool is_active() { return active; }
 private:
-  bool is_gen;
+  bool generator;
+  bool active;
 };
 
-typedef MAP<std::string, double> Attr;
+typedef UMAP<std::string, double> Attr;
 
 /** 
  *  Arrival process.
@@ -55,7 +59,7 @@ class Arrival: public Process {
     double activity;
     ArrTime(): start(-1), activity(0) {}
   };
-  typedef MAP<std::string, ArrTime> ResTime;
+  typedef UMAP<std::string, ArrTime> ResTime;
   
 public:
   /**
@@ -66,24 +70,27 @@ public:
    * @param first_activity  the first activity of a user-defined R trajectory
    */
   Arrival(Simulator* sim, std::string name, int mon, Activity* first_activity, Generator* gen):
-    Process(sim, name, mon), activity(first_activity), gen(gen) {}
+    Process(sim, name, mon), activity(first_activity), gen(gen), busy_until(-1), remaining(0) {}
   
+  void run();
   void activate();
+  void deactivate(bool restart);
   
   int set_attribute(std::string key, double value);
   inline Attr* get_attributes() { return &attributes; }
   
-  inline void set_start(double start) { lifetime.start = start; }
   inline void set_start(std::string name, double start) { restime[name].start = start; }
-  inline double get_start() { return lifetime.start; }
   inline double get_start(std::string name) { return restime[name].start; }
+  inline double get_start() { return lifetime.start; }
   
-  inline void set_activity(double act) { lifetime.activity = act; }
   inline void set_activity(std::string name, double act) { restime[name].activity = act; }
-  inline double get_activity() { return lifetime.activity; }
   inline double get_activity(std::string name) { return restime[name].activity; }
+  inline double get_activity() { return lifetime.activity; }
   
-  void leaving(std::string name, double time);
+  inline double get_remaining() { return remaining; }
+  
+  void leave(std::string name, double time);
+  void reject(double time);
 
 private:
   ArrTime lifetime;   /**< time spent in the whole trajectory */
@@ -91,6 +98,8 @@ private:
   Activity* activity; /**< current activity from an R trajectory */
   Generator* gen;     /**< parent generator */
   Attr attributes;    /**< user-defined (key, value) pairs */
+  double busy_until;  /**< next scheduled event time */
+  double remaining;   /**< time remaining in a deactivated arrival */
 };
 
 /**
@@ -120,7 +129,7 @@ public:
     attr_stats.clear();
   }
   
-  void activate();
+  void run();
   
   /**
    * Gather attribute statistics.
@@ -182,28 +191,55 @@ private:
   StatsMap attr_stats;      /**< attribute statistics */
 };
 
-struct RQItem {
+struct RSeize {
+  double arrived_at;
   Arrival* arrival;
   int amount;
   int priority;
-  double arrived_at;
+  int preemptible;
+  bool restart;
   
-  RQItem(Arrival* arrival, int amount, int priority, double arrived_at):
-    arrival(arrival), amount(amount), priority(priority), arrived_at(arrived_at) {}
-  
-  bool operator<(const RQItem& other) const {
-    if(priority == other.priority)
-      return arrived_at > other.arrived_at;
-    return priority < other.priority;
+  RSeize(double arrived_at, Arrival* arrival, int amount, int priority, 
+         int preemptible, bool restart):
+    arrived_at(arrived_at), arrival(arrival), amount(amount), priority(priority),
+    preemptible(preemptible), restart(restart) {}
+};
+
+struct RQComp {
+  bool operator()(const RSeize& lhs, const RSeize& rhs) const {
+    if (lhs.priority == rhs.priority) {
+      if (lhs.arrived_at == rhs.arrived_at)
+        return lhs.arrival->get_remaining() > rhs.arrival->get_remaining();
+      return lhs.arrived_at < rhs.arrived_at;
+    }
+    return lhs.priority > rhs.priority;
   }
 };
 
+struct RSCompFIFO {
+  bool operator()(const RSeize& lhs, const RSeize& rhs) const {
+    if (lhs.preemptible == rhs.preemptible)
+      return lhs.arrived_at < rhs.arrived_at;
+    return lhs.preemptible < rhs.preemptible;
+  }
+};
+
+struct RSCompLIFO {
+  bool operator()(const RSeize& lhs, const RSeize& rhs) const {
+    if (lhs.preemptible == rhs.preemptible)
+      return lhs.arrived_at > rhs.arrived_at;
+    return lhs.preemptible < rhs.preemptible;
+  }
+};
+
+typedef MSET<RSeize, RQComp> RPQueue;
+typedef MSET<RSeize, RSCompFIFO> FIFO;
+typedef MSET<RSeize, RSCompLIFO> LIFO;
+
 /** 
-*  Generic resource, a passive entity that comprises server + FIFO queue.
+*  Generic resource, a passive entity that comprises server + a priority queue.
 */
 class Resource: public Entity {
-  typedef PQUEUE<RQItem> RPQueue;
-  
 public:
   /**
   * Constructor.
@@ -225,10 +261,9 @@ public:
   void reset() {
     server_count = 0;
     queue_count = 0;
-    while (!queue.empty()) {
-      delete queue.top().arrival;
-      queue.pop();
-    }
+    foreach_(RPQueue::value_type& itr, queue)
+      delete itr.arrival;
+    queue.clear();
     res_stats.clear();
   }
   
@@ -240,7 +275,7 @@ public:
   * 
   * @return  SUCCESS, ENQUEUED, REJECTED
   */
-  int seize(Arrival* arrival, int amount, int priority);
+  int seize(Arrival* arrival, int amount, int priority, int preemptible, bool restart);
   
   /**
   * Release resources.
@@ -266,7 +301,7 @@ public:
   int get_server_count() { return server_count; }
   int get_queue_count() { return queue_count; }
   
-private:
+protected:
   int capacity;
   int queue_size;
   int server_count;     /**< number of arrivals being served */
@@ -274,13 +309,147 @@ private:
   RPQueue queue;        /**< queue container */
   StatsMap res_stats;   /**< resource statistics */
   
-  inline bool room_in_server(int amount) { 
-    if (capacity < 0) return 1;
-    return server_count + amount <= capacity; 
+  virtual inline bool room_in_server(int amount, int priority) {
+    if (capacity < 0) return true;
+    return server_count + amount <= capacity;
   }
-  inline bool room_in_queue(int amount) { 
-    if (queue_size < 0) return 1;
-    return queue_count + amount <= queue_size;
+  
+  virtual inline void insert_in_server(double time, Arrival* arrival, int amount, 
+                                       int priority, int preemptible, bool restart) {
+    server_count += amount;
+  }
+  
+  virtual inline void remove_from_server(Arrival* arrival, int amount) {
+    server_count -= amount;
+  }
+  
+  virtual inline bool room_in_queue(int amount, int priority) {
+    if (queue_size < 0) return true;
+    if (queue_count + amount <= queue_size) return true;
+    int count = 0;
+    foreach_r_ (RPQueue::value_type& itr, queue) {
+      if (priority > itr.priority)
+        count += itr.amount;
+      else break;
+      if (count >= amount) return true;
+    }
+    return false;
+  }
+  
+  virtual inline void insert_in_queue(double time, Arrival* arrival, int amount, 
+                              int priority, int preemptible, bool restart) {
+    if (queue_size > 0) while (queue_count + amount > queue_size) {
+      RPQueue::iterator last = --queue.end();
+      last->arrival->reject(time);
+      queue_count -= last->amount;
+      queue.erase(last);
+    }
+    queue_count += amount;
+    queue.emplace(time, arrival, amount, priority, preemptible, restart);
+  }
+  
+  virtual inline bool try_serve_from_queue(double time) {
+    RPQueue::iterator next = queue.begin();
+    if (room_in_server(next->amount, next->priority)) {
+      if (next->arrival->is_monitored()) {
+        double last = next->arrival->get_activity(this->name);
+        next->arrival->set_activity(this->name, time - last);
+      }
+      next->arrival->activate();
+      insert_in_server(next->arrived_at, next->arrival, next->amount,
+                       next->priority, next->preemptible, next->restart);
+      queue_count -= next->amount;
+      queue.erase(next);
+      return true;
+    }
+    return false;
+  }
+};
+
+/** 
+*  Preemptive resource.
+*/
+template <typename T>
+class PreemptiveResource: public Resource {
+public:
+  /**
+  * Constructor.
+  * @param preempt_order  "fifo" or "lifo"
+  */
+  PreemptiveResource(Simulator* sim, std::string name, int mon, int capacity, int queue_size):
+    Resource(sim, name, mon, capacity, queue_size) {}
+  
+  ~PreemptiveResource() { reset(); }
+  
+  void reset() {
+    Resource::reset();
+    server.clear();
+  }
+  
+protected:
+  RPQueue preempted;    /**< preempted arrivals */
+  T server;             /**< server container */
+  
+  virtual inline bool room_in_server(int amount, int priority) {
+    if (capacity < 0) return true;
+    if (server_count + amount <= capacity) return true;
+    int count = 0;
+    foreach_ (typename T::value_type& itr, server) {
+      if (priority > itr.preemptible)
+        count += itr.amount;
+      else break;
+      if (count >= amount) return true;
+    }
+    return false;
+  }
+  
+  virtual inline void insert_in_server(double time, Arrival* arrival, int amount, 
+                               int priority, int preemptible, bool restart) {
+    if (capacity > 0) while (server_count + amount > capacity) {
+      typename T::iterator first = server.begin();
+      first->arrival->deactivate(first->restart);
+      if (first->arrival->is_monitored()) {
+        double last = first->arrival->get_activity(this->name);
+        first->arrival->set_activity(this->name, time - last);
+      }
+      preempted.insert((*first));
+      queue_count += first->amount;
+      server_count -= first->amount;
+      server.erase(first);
+    }
+    server_count += amount;
+    server.emplace(time, arrival, amount, priority, preemptible, restart);
+  }
+  
+  virtual inline void remove_from_server(Arrival* arrival, int amount) {
+    typename T::iterator itr = server.begin();
+    while (itr->arrival != arrival) ++itr;
+    server.erase(itr);
+    server_count -= amount;
+  }
+  
+  virtual inline bool try_serve_from_queue(double time) {
+    RPQueue::iterator next;
+    bool flag = false;
+    if (!preempted.empty()) {
+      next = preempted.begin();
+      flag = true;
+    }
+    else next = queue.begin();
+    if (room_in_server(next->amount, next->priority)) {
+      if (next->arrival->is_monitored()) {
+        double last = next->arrival->get_activity(this->name);
+        next->arrival->set_activity(this->name, time - last);
+      }
+      next->arrival->activate();
+      insert_in_server(next->arrived_at, next->arrival, next->amount,
+                       next->priority, next->preemptible, next->restart);
+      queue_count -= next->amount;
+      if (flag) preempted.erase(next);
+      else queue.erase(next);
+      return true;
+    }
+    return false;
   }
 };
 
