@@ -15,6 +15,7 @@ test_that("an empty environment behaves as expected", {
 
 t0 <- create_trajectory("") %>%
   seize("server", 1) %>%
+  set_attribute("dummy", 1) %>%
   release("server", 1)
 
 test_that("the simulator is reset", {
@@ -24,27 +25,22 @@ test_that("the simulator is reset", {
     run(100) %>%
     reset()
   arrivals <- env%>%get_mon_arrivals()
+  arrivals_res <- env%>%get_mon_arrivals(TRUE)
   resources <- env%>%get_mon_resources()
+  attributes <- env%>%get_mon_attributes()
   
   expect_equal(env%>%now(), 0)
   expect_equal(env%>%peek(), 1)
   expect_equal(nrow(arrivals), 0)
+  expect_equal(nrow(arrivals_res), 0)
   expect_equal(nrow(resources), 0)
+  expect_equal(nrow(attributes), 0)
 })
 
 test_that("the simulator stops if there are no more events", {
-  onlyone <- function(){
-    generate <- 1
-    function() {
-      if (generate) {
-        generate <<- 0
-        return(1)
-      } else return(-1)
-    }
-  }
   env <- simmer() %>%
     add_resource("server", 1) %>%
-    add_generator("entity", t0, onlyone()) %>%
+    add_generator("entity", t0, at(1)) %>%
     run(10)
   
   expect_equal(env%>%now(), 1)
@@ -53,7 +49,7 @@ test_that("the simulator stops if there are no more events", {
 test_that("a negative simulation time is converted to positive", {
   env <- simmer() %>%
     add_resource("server", 1) %>%
-    add_generator("entity", t0, function() 1) %>%
+    add_generator("entity", t0, at(10)) %>%
     run(-10)
   
   expect_equal(env%>%now(), 10)
@@ -73,13 +69,18 @@ test_that("a stopped simulation can be resumed", {
 })
 
 test_that("there is verbose output", {
+  output <- paste0(".*(",
+    ".*sim.*dummy.*time: 1.*arrival0.*Seize.*server",
+    ".*sim.*dummy.*time: 1.*arrival0.*Release.*server",
+  ").*")
+  
   expect_output(
-  env <- simmer(verbose=TRUE) %>%
-    add_resource("server", 1) %>%
-    add_generator("entity", t0, function() 1) %>%
-    run(2),
-  "sim: anonymous | time: 1 | arrival: entity0 | activity: Seize(server)
-  sim: anonymous | time: 1 | arrival: entity0 | activity: Release(server)")
+    env <- simmer("dummy", verbose=TRUE) %>%
+      add_resource("server", 1) %>%
+      add_generator("arrival", t0, at(1)) %>%
+      run(),
+    output
+  )
 })
 
 test_that("we can force some errors (just to complete coverage)", {
@@ -88,7 +89,7 @@ test_that("we can force some errors (just to complete coverage)", {
   
   env <- simmer() %>% 
     add_resource("dummy") %>% 
-    add_generator("dummy", create_trajectory()%>%timeout(0), function() 1)
+    add_generator("dummy", create_trajectory()%>%timeout(0), function() 1, mon=1000)
   env$.__enclos_env__$private$sim_obj <- NULL
   
   expect_error(env %>% reset())
@@ -96,6 +97,7 @@ test_that("we can force some errors (just to complete coverage)", {
   expect_error(env %>% peek())
   expect_error(env %>% onestep())
   expect_error(env %>% get_mon_arrivals())
+  expect_error(env %>% get_mon_arrivals(TRUE))
   expect_error(env %>% get_mon_attributes())
   expect_error(env %>% get_mon_resources())
 })
