@@ -73,7 +73,7 @@ test_that("the trajectory stores the right number of activities", {
     ".*SetAttribute.*function", 
   ").*")
   
-  expect_output(t0, output)
+  expect_output(print(t0), output)
 })
 
 test_that("the head/tail pointers are correctly placed", {
@@ -95,10 +95,6 @@ test_that("the head/tail pointers are correctly placed", {
 })
 
 test_that("we can force some errors (just to complete coverage)", {
-  expect_warning(create_trajectory() %>% show_trajectory())
-  expect_warning(create_trajectory() %>% 
-                   timeout(1) %>% get_head() %>% show_activity())
-  
   expect_error(create_trajectory() %>% get_next_activity())
   expect_error(create_trajectory() %>% get_prev_activity())
   
